from unittest import TestCase
from unittest.mock import patch, MagicMock

from starlette.routing import Mount
from starlette.staticfiles import StaticFiles

from servey.servey_starlette.route_factory.static_site_route_factory import (
    StaticSiteRouteFactory,
)


class TestStaticSiteRouteFactory(TestCase):
    def test_create_routes(self):
        mock_directory = MagicMock()
        mock_directory.exists.return_value = True
        mock_directory.isdir.return_value = True
        route_factory = StaticSiteRouteFactory(path="/foobar", directory=mock_directory)

        with patch("os.path", mock_directory):
            routes = list(route_factory.create_routes())
            expected_routes = [
                Mount(
                    path="/foobar",
                    app=StaticFiles(directory=mock_directory, html=True),
                    name="static_site",
                )
            ]
            self.assertEqual(1, len(routes))
            self.assertEqual("/foobar", routes[0].path)
            self.assertEqual("static_site", routes[0].name)



import importlib
import os
from unittest import TestCase
from unittest.mock import patch
from uuid import UUID

from servey.action.action import action, get_action
from servey.servey_celery.celery_config.fixed_rate_trigger_config import (
    FixedRateTriggerConfig,
)
from servey.subscription.subscription import subscription
from servey.trigger.fixed_rate_trigger import FixedRateTrigger


class TestCeleryApp(TestCase):
    def test_app(self):
        @action(triggers=(FixedRateTrigger(10),))
        def ping() -> UUID:
            """No implementation required"""

        # noinspection PyUnusedLocal
        @action
        def consume_message(message: str):
            """No implementation required"""

        subscription_ = subscription(
            str,
            name="message_broadcaster",
            action_subscribers=(get_action(consume_message),),
        )

        with (
            patch(
                "servey.finder.action_finder_abc.find_actions",
                return_value=[get_action(ping), get_action(consume_message)],
            ),
            patch(
                "servey.servey_celery.celery_config.subscription_config.find_subscriptions",
                return_value=[subscription_],
            ),
            patch.dict(os.environ, dict(CELERY_BROKER="redis://localhost:6379/0")),
            patch("celery.app.task.Task.apply_async", return_value=None),
        ):
            _reload_module()
            # noinspection PyUnresolvedReferences
            app = _module.app
            next(t for t in app.tasks if t.endswith("test_celery_app.ping"))
            next(t for t in app.tasks if t.endswith("test_celery_app.consume_message"))
            subscription_.publish("Some message")

    def test_app_no_triggers(self):
        @action
        def pong() -> UUID:
            """No implementation required"""

        with (
            patch(
                "servey.finder.action_finder_abc.find_actions",
                return_value=[get_action(pong)],
            ),
            patch(
                "servey.servey_celery.celery_config.subscription_config.find_subscriptions",
                return_value=[],
            ),
            patch.dict(os.environ, dict(CELERY_BROKER="redis://localhost:6379/0")),
            patch("celery.app.task.Task.apply_async", return_value=None),
        ):
            _reload_module()
            # noinspection PyUnresolvedReferences
            app = _module.app
            self.assertIsNone(
                next((t for t in app.tasks if t.endswith("test_celery_app.pong")), None)
            )

    def test_on_after_connect(self):
        @action(triggers=(FixedRateTrigger(10),))
        def ping() -> UUID:
            """No implementation required"""

        class MockOnAfterConfigure:
            def __init__(self):
                self.fn = None

            def connect(self, fn):
                assert self.fn is None
                assert fn is not None
                self.fn = fn

        ns = {}

        class MockTask:
            @staticmethod
            def s():
                return "foobar"

        class MockCeleryApp:
            def __init__(self):
                self.task_called = 0
                self.periodic_called = 0
                self.on_after_configure = MockOnAfterConfigure()

            def task(self, fn):
                self.task_called += 1
                assert fn == ping
                return MockTask

            def add_periodic_task(self, interval, s):
                self.periodic_called += 1
                assert interval == 10
                assert s == "foobar"

        with patch(
            "servey.servey_celery.celery_config.fixed_rate_trigger_config.find_actions_with_trigger_type",
            return_value=[(get_action(ping), FixedRateTrigger(10))],
        ):
            app = MockCeleryApp()
            config = FixedRateTriggerConfig()
            config.configure(app, ns)
            self.assertEqual(app.task_called, 1)
            app.on_after_configure.fn(app)
            self.assertEqual(app.periodic_called, 1)


_module = None


def _reload_module():
    global _module
    if _module:
        importlib.reload(_module)
    else:
        from servey.servey_celery import celery_app

        _module = celery_app



from unittest import TestCase
from unittest.mock import patch

from servey.action.action import action, get_action
from servey.servey_web_page.redirect import Redirect
from servey.servey_web_page.web_page_event_handler import WebPageEventHandlerFactory
from servey.servey_web_page.web_page_trigger import WebPageTrigger


class TestWebPageEventHandler(TestCase):
    def test_event_handler(self):
        @action(triggers=WebPageTrigger())
        def add(a: int, b: int) -> int:
            return a + b

        with patch(
            "servey.servey_web_page.web_page_trigger.get_servey_main",
            return_value="tests.servey_web_page",
        ):
            factory = WebPageEventHandlerFactory()
            handler = factory.create(get_action(add))
            event = {"httpMethod": "GET", "queryStringParameters": {"a": "3", "b": "5"}}
            context = {}
            self.assertTrue(handler.is_usable(event, context))
            result = handler.handle(event, context)
            expected = {
                "statusCode": 200,
                "headers": {"Content-Type": "text/html"},
                "body": "The result was 8",
            }
            self.assertEqual(expected, result)

    def test_async(self):
        @action(triggers=WebPageTrigger())
        async def add(a: int, b: int) -> int:
            return a + b

        with patch(
            "servey.servey_web_page.web_page_trigger.get_servey_main",
            return_value="tests.servey_web_page",
        ):
            factory = WebPageEventHandlerFactory()
            handler = factory.create(get_action(add))
            event = {"httpMethod": "GET", "queryStringParameters": {"a": "3", "b": "5"}}
            context = {}
            self.assertTrue(handler.is_usable(event, context))
            result = handler.handle(event, context)
            expected = {
                "statusCode": 200,
                "headers": {"Content-Type": "text/html"},
                "body": "The result was 8",
            }
            self.assertEqual(expected, result)

    def test_redirect(self):
        @action(triggers=WebPageTrigger())
        def add(a: int, b: int) -> Redirect:
            return Redirect(f"https://foobar.com/{a+b}")

        factory = WebPageEventHandlerFactory()
        handler = factory.create(get_action(add))
        event = {"httpMethod": "GET", "queryStringParameters": {"a": "3", "b": "5"}}
        context = {}
        self.assertTrue(handler.is_usable(event, context))
        result = handler.handle(event, context)
        expected = {"statusCode": 307, "headers": {"Location": "https://foobar.com/8"}}
        self.assertEqual(expected, result)



from asyncio import get_event_loop
from http.client import HTTPException
from unittest import TestCase
from unittest.mock import patch

from jinja2 import Environment, PackageLoader

from servey.action.action import action, get_action
from servey.errors import ServeyError
from servey.servey_web_page.redirect import Redirect
from servey.servey_web_page.web_page_action_endpoint_factory import (
    WebPageActionEndpointFactory,
)
from servey.servey_web_page.web_page_trigger import WebPageTrigger
from servey.trigger.web_trigger import WebTriggerMethod
from tests.servey_starlette.action_endpoint.test_action_endpoint import build_request


class TestWebPageActionEndpoint(TestCase):
    def test_endpoint(self):
        @action(triggers=WebPageTrigger())
        def add(a: int, b: int) -> int:
            return a + b

        with patch(
            "servey.servey_web_page.web_page_action_endpoint.get_environment",
            return_value=Environment(
                loader=PackageLoader("tests.servey_web_page", "templates"),
                auto_reload=False,  # Uvicorn handles this
            ),
        ):
            factory = WebPageActionEndpointFactory()
            endpoint = factory.create(get_action(add), set(), [factory])
            self.assertEqual("add.j2", endpoint.template_name)
            schema = {}
            endpoint.to_openapi_schema(schema)
            self.assertEqual({}, schema)
            request = build_request(path="/add", query_string="a=3&b=5")
            loop = get_event_loop()
            result = loop.run_until_complete(endpoint.execute(request))
            self.assertEqual(b"The result was 8", result.body)

    def test_redirect(self):
        @action(triggers=WebPageTrigger())
        def add(a: int, b: int) -> Redirect:
            return Redirect(f"https://foobar.com/{a+b}")

        with patch(
            "servey.servey_web_page.web_page_action_endpoint.get_environment",
            return_value=Environment(
                loader=PackageLoader("tests.servey_web_page", "templates"),
                auto_reload=False,  # Uvicorn handles this
            ),
        ):
            factory = WebPageActionEndpointFactory()
            endpoint = factory.create(get_action(add), set(), [factory])
            request = build_request(path="/add", query_string="a=3&b=5")
            loop = get_event_loop()
            result = loop.run_until_complete(endpoint.execute(request))
            self.assertEqual(b"", result.body)
            self.assertEqual("https://foobar.com/8", result.headers["location"])

    def test_error(self):
        @action(triggers=WebPageTrigger())
        def add(a: int, b: int) -> int:
            return str(a + b)

        with patch(
            "servey.servey_web_page.web_page_action_endpoint.get_environment",
            return_value=Environment(
                loader=PackageLoader("tests.servey_web_page", "templates"),
                auto_reload=False,  # Uvicorn handles this
            ),
        ):
            factory = WebPageActionEndpointFactory()
            endpoint = factory.create(get_action(add), set(), [factory])
            request = build_request(path="/add", query_string="a=3&b=5")
            with self.assertRaises(HTTPException):
                loop = get_event_loop()
                loop.run_until_complete(endpoint.execute(request))

    def test_no_triggers(self):
        @action
        def add(a: int, b: int) -> int:
            return str(a + b)

        factory = WebPageActionEndpointFactory()
        result = factory.create(get_action(add), set(), [factory])
        self.assertIsNone(result)

    def test_multi_triggers(self):
        @action(
            triggers=(
                WebPageTrigger(WebTriggerMethod.GET),
                WebPageTrigger(WebTriggerMethod.POST),
            )
        )
        def add(a: int, b: int) -> int:
            return str(a + b)

        factory = WebPageActionEndpointFactory()
        with self.assertRaises(ServeyError):
            factory.create(get_action(add), set(), [factory])

    def test_no_result(self):
        @action(triggers=WebPageTrigger(WebTriggerMethod.GET))
        def dummy():
            pass

        with patch(
            "servey.servey_web_page.web_page_action_endpoint.get_environment",
            return_value=Environment(
                loader=PackageLoader("tests.servey_web_page", "templates"),
                auto_reload=False,  # Uvicorn handles this
            ),
        ):
            factory = WebPageActionEndpointFactory()
            endpoint = factory.create(get_action(dummy), set(), [factory])
            request = build_request(path="/dummy", query_string="a=3&b=5")

            loop = get_event_loop()
            result = loop.run_until_complete(endpoint.execute(request))
            self.assertEqual(b"No model was passed in here", result.body)



from unittest import TestCase

from dateutil.relativedelta import relativedelta

# noinspection PyProtectedMember
from servey.servey_strawberry.entity_factory.dataclass_factory import _UserCache


class TestDataclassFactory(TestCase):
    def test_user_cache(self):
        cache = _UserCache(relativedelta(seconds=10))
        cache.set("foo", "bar")
        self.assertEqual(cache.get("foo"), "bar")
        cache.delete("foo")
        self.assertIsNone(cache.get("foo"))
        cache.set("foo", "bar")
        cache.clear()
        self.assertIsNone(cache.get("foo"))



from dataclasses import dataclass
from unittest import TestCase
from unittest.mock import patch

from servey.action.action import action, get_action
from servey.errors import ServeyError
from servey.servey_aws.lambda_router import invoke
from servey.servey_aws.router.appsync_router import AppsyncRouter
from servey.trigger.web_trigger import WEB_GET


class TestLambdaInvoker(TestCase):
    def test_router(self):
        @action(triggers=WEB_GET)
        def echo_get(val: str) -> str:
            return val

        with patch(
            "servey.servey_aws.router.router.find_actions",
            return_value=[get_action(echo_get)],
        ):
            event = dict(action_name="echo_get", params=dict(val="foo"))
            result = invoke(event, None)
            expected_result = "foo"
            self.assertEqual(expected_result, result)

    def test_router_no_handler(self):
        @action(triggers=WEB_GET)
        def echo_get(val: str) -> str:
            """Dummy"""

        with patch(
            "servey.servey_aws.router.router.find_actions",
            return_value=[get_action(echo_get)],
        ), self.assertRaises(StopIteration):
            event = dict(action_name="not_existing")
            invoke(event, None)

    def test_router_no_action(self):
        @action(triggers=WEB_GET)
        def echo_get(val: str) -> str:
            """Dummy"""

        with patch(
            "servey.servey_aws.router.router.find_actions",
            return_value=[get_action(echo_get)],
        ), self.assertRaises(ServeyError):
            invoke({}, None)

    def test_router_api_gateway(self):
        @action(triggers=WEB_GET)
        def echo_get(val: str) -> str:
            return val

        with patch(
            "servey.servey_aws.router.router.find_actions",
            return_value=[get_action(echo_get)],
        ), patch(
            "servey.finder.action_finder_abc.find_actions",
            return_value=[get_action(echo_get)],
        ):
            event = dict(path="/actions/echo-get", params=dict(val="foo"))
            result = invoke(event, None)
            expected_result = "foo"
            self.assertEqual(expected_result, result)

    def test_router_appsync(self):
        @action(triggers=WEB_GET)
        def echo_get(val: str) -> str:
            return val

        with patch(
            "servey.servey_aws.router.router.find_actions",
            return_value=[get_action(echo_get)],
        ), patch(
            "servey.finder.action_finder_abc.find_actions",
            return_value=[get_action(echo_get)],
        ):
            event = dict(path="/actions/echo-get", params=dict(val="foo"))
            result = invoke(event, None)
            expected_result = "foo"
            self.assertEqual(expected_result, result)

    def test_nested_router_appsync(self):

        @dataclass
        class NumberStats:
            int_value: int

            @action
            def factorial(self) -> int:
                result_ = 1
                value = self.int_value
                while value > 1:
                    result_ *= value
                    value -= 1
                return result_

        @action(triggers=WEB_GET)
        def number_stats(int_value: int) -> NumberStats:
            return NumberStats(int_value)

        event = {
            "arguments": {},
            "identity": None,
            "source": {
                "intValue": 5,
            },
            "prev": None,
            "info": {
                "selectionSetList": [],
                "selectionSetGraphQL": "{}",
                "fieldName": "factorial",
                "parentTypeName": "NumberStats",
                "variables": {
                    "pageKey": None
                }
            },
            "stash": {}
        }
        action_ = get_action(number_stats)
        router = AppsyncRouter()
        setattr(router, '_web_trigger_actions', ((action_, action_.triggers[0]),))
        handler = router.create_handler(event, None)
        result = handler.handle(event, None)
        self.assertEqual(120, result)



from unittest import TestCase

from servey.action.action import action, get_action
from servey.servey_web_page.web_page_trigger import WebPageTrigger
from servey.servey_web_page.web_page_trigger_handler import WebPageTriggerHandler


class TestWebPageTriggerHandler(TestCase):
    def test_event_handler(self):
        @action(triggers=WebPageTrigger())
        def add(a: int, b: int) -> int:
            return a + b

        lambda_definition = {}
        handler = WebPageTriggerHandler()
        action_ = get_action(add)
        handler.handle_trigger(action_, action_.triggers[0], lambda_definition)
        expected = {
            "events": [{"http": {"cors": True, "method": "get", "path": "/add"}}]
        }
        self.assertEqual(expected, lambda_definition)
